using System;
using System.Data;

namespace SubSonic
{

    [Serializable]
    public class ActiveList<ItemType, ListType> : AbstractList<ItemType, ListType> 
        where ItemType : ActiveRecord<ItemType>, new()
        where ListType : AbstractList<ItemType, ListType>, new()
    {
        public ActiveList(){}

        public ActiveRecord<ItemType> Find(object primaryKeyValue) {
            ActiveRecord<ItemType> result = null;
            foreach (ItemType item in this) {
                if (item.GetPrimaryKeyValue().Equals(primaryKeyValue)) {
                    result = item;
                }
            }
            return result;
        }

        /// <summary>
        /// Creates and loads the collection, leaving the IDataReader open.
        /// </summary>
        /// <param name="rdr"></param>
        public ActiveList(IDataReader rdr) 
        {
            Load(rdr);
        }

        /// <summary>
        /// Creates and loads the collection, with option to set IDataReader close behavior.
        /// </summary>
        /// <param name="rdr"></param>
        /// <param name="closeReader"></param>
        public ActiveList(IDataReader rdr, bool closeReader) 
        {
            Load(rdr);
            if (closeReader)
            {
                rdr.Close();
            }
        }

        /// <summary>
        /// Saves all the records in a collection to the DB
        /// </summary>
        public void SaveAll(string userName) {
            foreach (ItemType item in this) {
                item.Save(userName);
            }
        }
        /// <summary>
        /// Saves all the records in a collection to the DB
        /// </summary>
        public void SaveAll(Guid userID) {
            foreach (ItemType item in this) {
                item.Save(userID);
            }
        }
        /// <summary>
        /// Saves all the records in a collection to the DB
        /// </summary>
        public void SaveAll() {
            foreach (ItemType item in this) {
                item.Save();
            }
        }

        /// <summary>
        /// Uses a transaction to quickly saves all the records in a collection to the DB
        /// The primary keys of any new records are not updated in the collection
        /// You must reload the collection to get the latest data
        /// </summary>
        public void BatchSave(string userName)
        {
            QueryCommandCollection commands = new QueryCommandCollection();
            foreach (ItemType item in this)
            {
                QueryCommand cmd = item.GetSaveCommand(userName);
                if (cmd != null)
                    commands.Add(cmd);
            }

            DataService.ExecuteTransaction(commands);
        }

        public void BatchSave()
        {
            BatchSave(string.Empty);
        }
        public void BatchSave(Guid userID)
        {
            BatchSave(userID.ToString());
        }
        public void BatchSave(int userID)
        {
            BatchSave(userID.ToString());
        }

        /// <summary>
        /// Saves all the records in a collection to the DB
        /// </summary>
        public void SaveAll(int userId) {
            foreach (ItemType item in this) {
                item.Save(userId);
            }
        }

        public ListType Clone()
        {
            ListType coll = new ListType();
            foreach(ItemType item in this)
            {
                ItemType newItem = item.Clone();
                coll.Add(newItem);
            }
            return coll;
        }
    }
}
