using System.CodeDom;
using System.Diagnostics;
using System.IO;
using System.Text;
using System.Web.Compilation;
using System.Web.Hosting;
using SubSonic.Utilities;
using SubSonic.CodeGenerator;

namespace SubSonic
{
    /// <summary>
    /// The Magic...
    /// </summary>
    public class BuildProvider : System.Web.Compilation.BuildProvider
    {
        
        /// <summary>
        /// This is the main method used by the build provider
        /// </summary>
        /// <param name="assemblyBuilder"></param>
        public override void GenerateCode(AssemblyBuilder assemblyBuilder)
        {
            Utility.WriteTrace("Invoking BuildProvider");
            DataService.LoadProviders();
            ICodeLanguage language = new CSharpCodeLanguage();
            
            DirectoryInfo di = new DirectoryInfo(HostingEnvironment.ApplicationPhysicalPath + "\\App_Code");
            FileInfo[] fi = di.GetFiles("*.vb");
            if(fi.Length > 0)
            {
                language = new VBCodeLanguage();
            }
            //Debug.Assert(isVB, "VB");
            StringBuilder code = new StringBuilder(language.DefaultUsingStatements);

            TurboCompiler tc = new TurboCompiler();
            //loop over the providers, and generate code for each
            foreach(DataProvider provider in DataService.Providers)
            {
                Utility.WriteTrace("Creating code for " + provider.Name);
                string[] tableList = DataService.GetTableNames(provider.Name);
                string[] viewList = DataService.GetViewNames(provider.Name);


                foreach(string tbl in tableList)
                {
                    TurboTemplate tt = CodeService.BuildClassTemplate(tbl, language, provider);
                    tc.AddTemplate(tt);
                    TurboTemplate tODS = CodeService.BuildODSTemplate(tbl, language, provider);
                    tc.AddTemplate(tODS);
                }

                foreach(string view in viewList)
                {
                    TurboTemplate tt = CodeService.BuildViewTemplate(view, language, provider);
                    tc.AddTemplate(tt);
                }

                tc.AddTemplate(CodeService.BuildSPTemplate(language, provider));
            }

            if(DataService.Providers.Count > 0)
            {
                tc.AddTemplate(CodeService.BuildStructsTemplate(language, DataService.Provider));
            }

            foreach (TurboTemplate tt in tc.Templates)
            {
                tt.AddUsingBlock = false;
            }
            tc.Run();
            foreach(TurboTemplate tt in tc.Templates)
            {
                code.Append(tt.FinalCode);
            }
            assemblyBuilder.AddCodeCompileUnit(this, new CodeSnippetCompileUnit(code.ToString()));
        }
    }
}