using System;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Data;

namespace SubSonic
{

    //Thank you photoz!

    [ToolboxData("<{0}:RadioButtons runat=server></{0}:RadioButtons>")]
    public class RadioButtons : RadioButtonList
    {
        private string promptText = "None";
        public string PromptText
        {
            get { return promptText; }
            set { promptText = value; }
        }

        private string promptValue = string.Empty;
        public string PromptValue
        {
            get { return promptValue; }
            set { promptValue = value; }
        }

        private bool showPrompt;
        public bool ShowPrompt
        {
            get { return showPrompt; }
            set { showPrompt = value; }
        }

        private string providerName = string.Empty;
        public string ProviderName
        {
            get { return providerName; }
            set { providerName = value; }
        }

        private string tableName = string.Empty;
        public string TableName
        {
            get { return tableName; }
            set { tableName = value; }
        }

        private string textField = string.Empty;
        public string TextField
        {
            get { return textField; }
            set { textField = value; }
        }

        private string valueField = string.Empty;
        public string ValueField
        {
            get { return valueField; }
            set { valueField = value; }
        }

        private string orderField = string.Empty;
        public string OrderField
        {
            get { return orderField; }
            set { orderField = value; }
        }

        protected override void OnInit(EventArgs e)
        {
            base.OnInit(e);


            if (!DesignMode)
            {
                //load em up
                //cheap way to check for load state
                if (Items.Count == 0)
                {
                    if (!String.IsNullOrEmpty(tableName))
                    {
                        Query q = new Query(tableName, providerName);

                        if (String.IsNullOrEmpty(valueField) || String.IsNullOrEmpty(textField))
                        {
                            //look it up using the table schema
                            TableSchema.Table tbl = DataService.GetSchema(tableName, providerName, TableType.Table);
                            if (tbl != null)
                            {
                                //set the fields accordingly
                                if (String.IsNullOrEmpty(valueField))
                                {
                                    valueField = tbl.PrimaryKey.ColumnName;
                                }

                                if (String.IsNullOrEmpty(textField))
                                {
                                    if (tbl.Columns.Count > 1)
                                    {
                                        textField = tbl.Columns[1].ColumnName;
                                    }
                                    else
                                    {
                                        textField = tbl.Columns[0].ColumnName;
                                    }
                                }
                            }
                            else
                            {
                                throw new Exception("Table name '" + tableName + "' using Provider '" + providerName + "' doesn't work");
                            }
                        }

                        q.SelectList = valueField + "," + textField;
                        if (!String.IsNullOrEmpty(OrderField))
                            q.OrderBy = OrderBy.Asc(OrderField);
                        else
                            q.OrderBy = OrderBy.Asc(textField);
                        //fire up a reader
                        IDataReader rdr = null;
                        try
                        {
                            rdr = q.ExecuteReader();
                            while (rdr.Read())
                            {
                                ListItem item = new ListItem(rdr[1].ToString(), rdr[0].ToString());
                                Items.Add(item);
                            }
                        }
                        catch (DataException x)
                        {
                            throw new Exception("Error loading up ListItems for " + ClientID + ": " + x.Message);
                        }
                        finally
                        {
                            if (rdr != null)
                            {
                                rdr.Close();
                            }
                        }
                        ListItem prompt = new ListItem(promptText, PromptValue);
                        if (showPrompt)
                            Items.Insert(0, prompt);
                    }
                    else
                    {
                        //do nothing
                    }
                }
            }
        }
    }
}
