using System;
using System.Collections.Generic;
using SubSonic.Utilities;
using System.Data;

namespace SubSonic
{

    public class DBCollection : List<DB>
    {

        public DB this[string dbName]
        {
            get
            {
                foreach (DB db in this)
                {
                    if (Utility.IsMatch(db.ProviderName, dbName))
                    {
                        return db;
                    }
                }
                return null;
            }
            set
            {
                for(int index = 0; index < Count; index++)
                {
                    if(Utility.IsMatch(this[index].ProviderName, dbName))
                    {
                        this[index] = value;
                        break;
                    }
                }
            }
        }

    }

    public class DB
    {

        public DB(string providerName)
        {
            //load up the tables
            Tables = new TableSchema.TableCollection();
            Views = new TableSchema.TableCollection();

            _tables.AddRange(DataService.GetTables(providerName));
            _views.AddRange(DataService.GetViews(providerName));

            if (_sps == null)
            {
                LoadStoredProcedures(providerName);
            }
        }

        private void LoadStoredProcedures(string providerName)
        {
            //DataService.ResetDatabases();
            string[] sps = DataService.GetSPList(providerName);
            if (sps.Length > 0)
            {
                _sps = new List<StoredProcedure>();
                //System.Data.IDataReader rdr = null;
                DataProvider provider = DataService.Providers[providerName];
                foreach (string s in sps)
                {
                    if(String.IsNullOrEmpty(provider.SPStartsWith) || s.StartsWith(provider.SPStartsWith))
                    {
                        //declare the sp
                        StoredProcedure sp = new StoredProcedure(s, provider);

                        //get the params
                        IDataReader rdr = DataService.GetSPParams(s, providerName);
                        while(rdr.Read())
                        {
                            StoredProcedure.Parameter par = new StoredProcedure.Parameter();
                            par.DBType = DataService.GetDbType(rdr[SqlSchemaVariable.DATA_TYPE].ToString(), providerName);
                            string sMode = rdr[SqlSchemaVariable.MODE].ToString();
                            if(Utility.IsMatch(sMode, SqlSchemaVariable.MODE_INOUT))
                            {
                                par.Mode = ParameterDirection.InputOutput;
                            }
                            par.Name = rdr[SqlSchemaVariable.NAME].ToString();
                            par.QueryParameter = provider.MakeParam(par.Name);
                            par.DisplayName = Utility.GetParameterName(par.Name, provider);
                            sp.Parameters.Add(par);
                        }
                        rdr.Close();
                        _sps.Add(sp);
                    }
                }
            }
        }
        #region props

        private string _connectionString;

        public string ConnectionString
        {
            get { return _connectionString; }
            set { _connectionString = value; }
        }

        private string _providerName;

        public string ProviderName
        {
            get { return _providerName; }
            set { _providerName = value; }
        }

        private DataProvider _provider;

        public DataProvider Provider
        {
            get { return _provider; }
            set { _provider = value; }
        }

        private TableSchema.TableCollection _tables;

        public TableSchema.TableCollection Tables
        {
            get { return _tables; }
            set { _tables = value; }
        }

        private TableSchema.TableCollection _views;

        public TableSchema.TableCollection Views
        {
            get { return _views; }
            set { _views = value; }
        }

        private List<StoredProcedure> _sps;// =new List<StoredProcedure>();

        public List<StoredProcedure> StoredProcedures {
            get
            {
                if(_sps == null)
                {
                    LoadStoredProcedures(ProviderName);
                }
                return _sps;
            }
            set { _sps = value; }
        }
	

        #endregion
    }
}
