///////////////////////////////////////////////////////
// Code author: Martin Lapierre, http://devinstinct.com
///////////////////////////////////////////////////////

using System;
using System.CodeDom;
using System.IO;
using System.Reflection;
using System.Runtime.InteropServices;
using Microsoft.CustomTool;
using SubSonic;

namespace SubSonic.Tools
{
	/// <summary>
	/// This is the SubSonic Custom Tool.
	/// </summary>
	[ComVisible(true)]
	[Guid("0C5AC1DF-1771-40f0-B31C-81A127CA531B")]
	public class SubSonicCustomTool
	   : BaseCodeGeneratorWithSite
	{
		#region Methods

		/// <summary>
		/// Does the code generation.
		/// </summary>
		/// <param name="inputFileName">The name of the input file.</param>
		/// <param name="inputFileContent">unused</param>
		/// <returns>The generated code bytes.</returns>
		protected override byte[] GenerateCode(string inputFileName, string inputFileContent)
		{
			try
			{
				SubSonicHost host = new SubSonicHost(inputFileName);
				CodeCompileUnit unit = host.SubSonic.GenerateSingleUnit(GetLanguage());

				string code = null;
				if (unit is CodeSnippetCompileUnit) // Assumes already in a string form.
					code = (unit as CodeSnippetCompileUnit).Value;
				else // Must write the DOM objects to a string.
					using (StringWriter writer = new StringWriter())
					{
						CodeProvider.GenerateCodeFromCompileUnit(unit, writer, null);
						code = writer.ToString();
					}
				return System.Text.Encoding.ASCII.GetBytes(code);
			}
			catch (Exception ex)
			{
				if (ex is TargetInvocationException & ex.InnerException != null)
					throw ex.InnerException;
				else
					throw;
			}
		}

		/// <summary>
		/// Gets the language to generate the code for.
		/// </summary>
		/// <returns>The language to generate the code for.</returns>
		protected LanguageType GetLanguage()
		{
			switch (CodeProvider.FileExtension)
			{
				case "cs":
					return LanguageType.CSharp;
				case "vb":
					return LanguageType.VB;
				default:
					throw new Exception(string.Format("The SubSonic Custom Tool doesn't generate '{0}' files.", CodeProvider.FileExtension));
			}
		}

		#endregion Methods
	}
}
