''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Code author: Martin Lapierre, http://devinstinct.com
''''''''''''''''''''''''''''''''''''''''''''''''''''''

Imports System.Collections.Generic
Imports System.Windows.Forms
Imports SubSonic.Tools

Public Class GenerationForm

	Private _isInitializing As Boolean = False
	Private _mustGenerate As Boolean = False
	Private _host As SubSonicHost = Nothing
	Private _providerData As New Dictionary(Of String, ProviderData)
	Private _currentProviderName As String = Nothing

	Protected SC_CLOSE As Integer = 61536
	Protected WM_SYSCOMMAND As Integer = 274


	Protected Overrides Sub WndProc(ByRef message As Message)

		If (message.Msg = WM_SYSCOMMAND And message.WParam.ToInt32() = SC_CLOSE) Then
			ValidateClose()
		Else
			MyBase.WndProc(message)
		End If

	End Sub

	Public Sub New(ByVal host As SubSonicHost)

		' This call is required by the Windows Form Designer.
		InitializeComponent()

		' Add any initialization after the InitializeComponent() call.
		_host = host
	End Sub

	Public ReadOnly Property MustGenerate() As Boolean
		Get
			Return _mustGenerate
		End Get
	End Property

	Public ReadOnly Property ProviderData() As Dictionary(Of String, ProviderData)
		Get
			Return _providerData
		End Get
	End Property

	Private Sub GeneratorForm_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
		Try
			Dim providerNames As String() = _host.SubSonic.GetProviderNames()

			' Init providers data
			_currentProviderName = providerNames(0)
			For Each providerName As String In providerNames
				Dim newProviderData As New ProviderData()
				newProviderData.Tables = _host.SubSonic.GetTables(providerName)
				newProviderData.Views = _host.SubSonic.GetViews(providerName)
				newProviderData.SPs = _host.SubSonic.GetSPs(providerName)
				_providerData.Add(providerName, newProviderData)
			Next

			' Init controls
			_isInitializing = True
			Me._providerList.DataSource = providerNames
			Me._selectTables.Activated = AddressOf OnProviderMustGenerate
			Me._selectViews.Activated = AddressOf OnProviderMustGenerate
			Me._selectSPs.Activated = AddressOf OnProviderMustGenerate
			If (providerNames.Length = 1) Then
				Me._useProviderBox.Checked = True
			End If
			InitCurrentProviderControls()
			_isInitializing = False

			Me.Activate()
		Catch ex As Exception
			' Must not throw from here.
			SubSonicEnvDTE.DisplayMessage(ex)
			Me.Close()
		End Try
	End Sub

	Private Sub InitCurrentProviderControls()

		Dim currentProviderData As ProviderData = _providerData(_currentProviderName)

		Me._useProviderBox.Checked = currentProviderData.MustGenerate
		Me._selectTables.InitCheckListBox(currentProviderData.Tables, currentProviderData.SelectedTables)
		Me._selectViews.InitCheckListBox(currentProviderData.Views, currentProviderData.SelectedViews)
		Me._selectSPs.InitCheckListBox(currentProviderData.SPs, currentProviderData.SelectedSPs)
		Me._addDependenciesBox.Checked = currentProviderData.AddDependencies

	End Sub

	Private Sub CollectCurrentProviderData()
		Dim currentProviderData As ProviderData = _providerData(_currentProviderName)

		currentProviderData.MustGenerate = Me._useProviderBox.Checked
		currentProviderData.SelectedTables = Me._selectTables.GetSelection()
		currentProviderData.SelectedViews = Me._selectViews.GetSelection()
		currentProviderData.SelectedSPs = Me._selectSPs.GetSelection()
		currentProviderData.AddDependencies = Me._addDependenciesBox.Checked
	End Sub

	Private Sub OK_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles OK.Click

		Dim selectedProviders As String = Nothing

		CollectCurrentProviderData()

		For Each dataEntry As KeyValuePair(Of String, ProviderData) In _providerData
			If dataEntry.Value.MustGenerate Then
				If selectedProviders Is Nothing Then
					selectedProviders = ControlChars.CrLf & ControlChars.CrLf & ControlChars.Tab & dataEntry.Key
				Else
					selectedProviders += ControlChars.CrLf & ControlChars.Tab & dataEntry.Key
				End If
			End If
		Next

		If (selectedProviders Is Nothing) Then
			Select Case MessageBox.Show("No provider selected for code generation." & ControlChars.CrLf & "Are you sure you want to cancel code generation?", "SubSonic", MessageBoxButtons.YesNo, MessageBoxIcon.Warning)
				Case Windows.Forms.DialogResult.Yes
					_mustGenerate = False
					Me.Close()
			End Select
		Else
			Select Case MessageBox.Show(String.Format("Please confirm code generation for the following provider(s): {0}", selectedProviders), "SubSonic", MessageBoxButtons.OKCancel, MessageBoxIcon.Information)
				Case Windows.Forms.DialogResult.OK
					_mustGenerate = True
					Me.Close()
			End Select
		End If
	End Sub

	Private Sub ValidateClose()
		Select Case MessageBox.Show("Are you sure you want to cancel code generation?", "SubSonic", MessageBoxButtons.YesNo, MessageBoxIcon.Question)
			Case Windows.Forms.DialogResult.Yes
				_mustGenerate = False
				Me.Close()
		End Select
	End Sub

	Private Sub Cancel_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Cancel.Click
		ValidateClose()
	End Sub

	Private Sub ProviderList_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles _providerList.SelectedIndexChanged, _providerList.SelectedIndexChanged
		If (Not _isInitializing) Then
			_isInitializing = True
			CollectCurrentProviderData()
			_currentProviderName = Me._providerList.SelectedValue
			InitCurrentProviderControls()
			_isInitializing = False
		End If
	End Sub

	Private Sub _selectTables_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs)
		If (Not _isInitializing) Then
			Me._useProviderBox.Checked = True
		End If
	End Sub

	Private Sub _selectViews_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs)
		If (Not _isInitializing) Then
			Me._useProviderBox.Checked = True
		End If
	End Sub

	Private Sub OnProviderMustGenerate()
		Me._useProviderBox.Checked = True
	End Sub

End Class

Public Class ProviderData
	Public MustGenerate As Boolean
	Public Tables As String()
	Public Views As String()
	Public SPs As String()
	Public SelectedTables As String()
	Public SelectedViews As String()
	Public SelectedSPs As String()
	Public AddDependencies As Boolean = True
End Class
