///////////////////////////////////////////////////////
// Code author: Martin Lapierre, http://devinstinct.com
///////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.CodeDom;
using SubSonic;

namespace SubSonic.Tools
{
    /// <summary>
    /// Legacy from an older SubSonic version.
    /// </summary>
    public enum LanguageType
    {
        CSharp,
        VB
    }

	/// <summary>
	/// Provides the features required for code generation. 
	/// </summary>
	public interface ISubSonic
	{
		#region Methods

		/// <summary>
		/// Generates the DAL in a single CodeCompileUnit from an input file.
		/// </summary>
		/// <param name="languageType">The language to generate the DAL for.</param>
		/// <returns>A single CodeCompileUnit.</returns>
		CodeCompileUnit GenerateSingleUnit(LanguageType languageType);

		/// <summary>
		/// Generates the DAL in multiple CodeCompileUnits from an input file.
		/// </summary>
		/// <param name="languageType">The language to generate the DAL for.</param>
		/// <returns>A dictionary of suggested filenames/CodeCompileUnit pairs.</returns>
		IDictionary<string, CodeCompileUnit> GenerateMultipleUnits(LanguageType languageType);

		/// <summary>
		/// Generates the DAL in multiple CodeCompileUnits for specific tables, views
		/// and, optionnaly, all stored procedures.
		/// </summary>
		/// <param name="languageType">The language to generate the DAL for.</param>
		/// <param name="tables">
		/// The list of tables to generate the DAL for; first element as "*" for all tables.
		/// Can be null or empty. A null value generates no table and no table struct. 
		/// <param name="views">
		/// The list of views to generate the DAL for; first element as "*" for all views.
		/// Can be null or empty. A null value generates no view and no view struct. 
		/// </param>
		/// <param name="sps">
		/// The list of stored procedures to generate the DAL for; first element as "*" for all stored procedures.
		/// Can be null or empty. A null value generates no stored procedures. 
		/// </param>
		/// <param name="providerName">
		/// The name of the provider to generate the DAL for.
		/// If null, generates code for all the providers.
		/// </param>
		/// <param name="addDependencies">Indicates whether to add the dependent tables or not.</param>
		/// <returns>A dictionary of suggested filenames/CodeCompileUnit pairs.</returns>
		IDictionary<string, CodeCompileUnit> GenerateMultipleUnits(LanguageType languageType, string[] tables, string[] views, string[] sps, bool addDependencies, string providerName);

		/// <summary>
		/// Gets the list of tables.
		/// </summary>
		/// <param name="providerName">
		/// The name of the provider to get the tables for.
		/// If null, uses the default provider.
		/// </param>
		/// <returns>A list of table names.</returns>
		string[] GetTables(string providerName);

		/// <summary>
		/// Gets the list of views.
		/// </summary>
		/// <param name="providerName">
		/// The name of the provider to get the views for.
		/// If null, uses the default provider.
		/// </param>
		/// <returns>A list of view names.</returns>
		string[] GetViews(string providerName);

		/// <summary>
		/// Gets the list of stored procedures.
		/// </summary>
		/// <param name="providerName">
		/// The name of the provider to get the stored procedures for.
		/// If null, uses the default provider.
		/// </param>
		/// <returns>A list of stored procedure names.</returns>
		string[] GetSPs(string providerName);

		/// <summary>
		/// Gets the list of provider names for the current application context.
		/// </summary>
		/// <returns>A list of provider names.</returns>
		string[] GetProviderNames();

		#endregion Methods
	}
}
